
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef AGGR_STAT_H
#define AGGR_STAT_H

#include "base/OLog.h"
#include "base/ILog.h"

class ostream;

// simple aggregate statisitcs

class AggrStat {
	public:
		typedef int Val;

	public:
		AggrStat();

		void reset();
		void record(Val val);
		void store(OLog &log) const;
		void load(ILog &log);

		AggrStat &operator +=(const AggrStat &s);

		int count() const { return theCount; }
		Val min() const { return theMin; }
		Val max() const { return theMax; }
		double sum() const { return theSum; }
		double mean() const { return theCount ? theSum/theCount : -1; }
		double stdDev() const;
		double relDevp() const;

		ostream &print(ostream &os, const String &pfx) const;

	protected:
		int theCount;
		Val theMax;
		Val theMin;
		double theSum;
		double theSqSum;
};

inline
ostream &operator <<(ostream &os, const AggrStat &s) { return s.print(os, ""); }

inline
OLog &operator <<(OLog &log, const AggrStat &s) { s.store(log); return log; }

inline
ILog &operator >>(ILog &log, AggrStat &s) { s.load(log); return log; }

#endif
