
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_OBJECT_IDENTIFIER_CLASS_H
#define PG_OBJECT_IDENTIFIER_CLASS_H

#include "base/UniqId.h"

class OidRec;

// a set of attributes that uniquely identifies a Polygraph object
// XXX: eliminate old OidRec
class ObjId {
	public:
		ObjId() { reset(); }
		ObjId(const OidRec &r);

		void reset();

		const UniqId &world() const { return theWorld; }
		int server() const { return theServer; }
		int type() const { return theType; }
		int name() const { return theName; }

		operator void*() const { return theName > 0 ? (void*)-1 : 0; }

		void world(const UniqId &aWorld) { theWorld = aWorld; }
		void server(int aServer) { theServer = aServer; }
		void type(int aType) { theType = aType; }
		void name(int aName) { theName = aName; }

		int hash() const;

		inline bool operator <(const ObjId &id) const;  // arbitrary order
		inline bool operator ==(const ObjId &id) const;
		inline bool operator !=(const ObjId &id) const;

		void store(OLog &ol) const;
		void load(ILog &il);

	protected:
		UniqId theWorld;   // world id (server or client)
		int theServer;     // host address index
		int theType;       // object content type
		int theName;       // object "name" (unique within a world and type)
};

inline
bool ObjId::operator <(const ObjId &id) const {
	if (theName < id.theName)
		return true;
	if (theName > id.theName)
		return false;

	if (theServer < id.theServer)
		return true;
	if (theServer > id.theServer)
		return false;
	
	return theWorld < id.theWorld;
}

inline
bool ObjId::operator ==(const ObjId &id) const {
	return 
		theName == id.theName &&
		theServer == id.theServer &&
		theWorld == id.theWorld;
}

inline
bool ObjId::operator !=(const ObjId &id) const {
	return !(*this == id);
}

inline
OLog &operator <<(OLog &ol, const ObjId &r) {
	r.store(ol);
	return ol;
}

inline
ILog &operator >>(ILog &il, ObjId &r) {
	r.load(il);
	return il;
}

#endif
