
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "StatPhaseRec.h"
#include "OLog.h"
#include "ILog.h"


StatPhaseRec::StatPhaseRec() {
}

OLog &StatPhaseRec::store(OLog &log) const {
	StatIntvlRec::store(log);
	return log 
		<< theName 
		<< theGoal
		<< theContType
		<< theConnClose
		<< theSockRdSzH << theSockWrSzH
		<< theBasicXacts << theImsXacts << theReloadXacts
		<< theIcpXacts
		<< theRptmVsSz << theConnLifeVsUse
		;
}

ILog &StatPhaseRec::load(ILog &log) {
	StatIntvlRec::load(log);
	return log
		>> theName
		>> theGoal
		>> theContType
		>> theConnClose
		>> theSockRdSzH >> theSockWrSzH
		>> theBasicXacts >> theImsXacts >> theReloadXacts
		>> theIcpXacts
		>> theRptmVsSz >> theConnLifeVsUse
		;
}

void StatPhaseRec::concat(const StatPhaseRec &r) {
	if (theName && r.theName)
		theName += "|" + r.theName;
	StatIntvlRec::concat(r);
	theGoal.concat(r.theGoal);
	join(r);
}

void StatPhaseRec::merge(const StatPhaseRec &r) {
	if (theName && r.theName)
		theName += "&" + r.theName;
	StatIntvlRec::merge(r);
	theGoal.merge(r.theGoal);
	join(r);
}

void StatPhaseRec::join(const StatPhaseRec &r) {
	if (!theName)
		theName = r.theName;

	theContType.add(r.theContType);
	theConnClose.add(r.theConnClose);
	theSockRdSzH.add(r.theSockRdSzH);
	theSockWrSzH.add(r.theSockWrSzH);
	theBasicXacts += r.theBasicXacts;
	theImsXacts += r.theImsXacts;
	theReloadXacts += r.theReloadXacts;
	theIcpXacts  += r.theIcpXacts;
	theRptmVsSz.add(r.theRptmVsSz);
	theConnLifeVsUse.add(r.theConnLifeVsUse);
}

ostream &StatPhaseRec::print(ostream &os, const String &pfx) const {
	os << "name:\t " << theName << endl;

	StatIntvlRec::print(os, pfx);

	theGoal.print(os, pfx + "goal.");

	TmSzHistStat imsAll;
	imsAll += theImsXacts.hits();
	imsAll += theImsXacts.misses();

	TmSzHistStat basicAll;
	basicAll += theBasicXacts.hits();
	basicAll += theBasicXacts.misses();

	TmSzHistStat repAll;
	repAll += imsAll;
	repAll += basicAll;
	repAll += theReloadXacts;

	repAll.print(os, pfx + "rep.");
	basicAll.print(os, pfx + "basic.");
	imsAll.print(os, pfx + "ims.");
	theReloadXacts.print(os, pfx + "reload.");
	theBasicXacts.print(os, "hit", "miss", pfx);
	theImsXacts.print(os, "sc200", "sc304", pfx + "ims.");

	theIcpXacts.print(os, "hit", "miss",  pfx + "icp.");

	theContType.print(os, pfx + "cont_type.");
	theConnClose.print(os, pfx + "conn_close.");
	theSockRdSzH.print(os, pfx + "so_read.size.");
	theSockWrSzH.print(os, pfx + "so_write.size.");
	theRptmVsSz.print(os, pfx + "rptm_vs_size.");
	theConnLifeVsUse.print(os, pfx + "conn_ttl_vs_use.");

	return os;
}
