
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "polygraph.h"

#include "Client.h"
#include "CltXact.h"
#include "runtime/ErrorMgr.h"
#include "IcpMsg.h"
#include "IcpClient.h"
#include "IcpCltXact.h"

#include "runtime/polyBcastChannels.h"
#include "runtime/polyErrors.h"
#include "polyLogCats.h"


Time IcpCltXact::TheTimeout;

void IcpCltXact::reset() {
	Assert(!thePendAlarmCnt);
	IcpXaction::reset();
	IcpCltUser::reset();

	theOwner = 0;
	theAddr = NetAddr();
	theOid.reset();
	theStartTime = theLifeTime = Time();
	theRepSize = Size();
	theResult = icpInvalid;
	theReason = 0;
	theLogCat = lgcCltSide;
}

void IcpCltXact::exec(Client *anOwner, const NetAddr &anAddr) {
	theOwner = anOwner;
	theAddr = anAddr;
	theOid = theReason->oidRec();
	Assert(icpClient());

	IcpXaction::start();

	theReserv = icpClient()->expectReply(this);

	IcpMsg r;
	r.opCode(icpQuery);
	r.oid(theOid);
	r.reqNum(theReserv.reqNum());
	r.peer(theAddr);

	if (!r.send(icpClient()->socket()))
		finish(errno);

	if (TheTimeout >= 0)
		sleepFor(TheTimeout);
}

void IcpCltXact::wakeUp(const Alarm &a) {
	AlarmUser::wakeUp(a);
	cerr << here << "ICP query to " << theAddr << " timedout after " << TheTimeout << endl;
	theResult = icpTimeout;
	finish(0);
}

void IcpCltXact::noteReply(const IcpMsg &r) {
	theResult = r.opCode();
	theRepSize = r.size();
	switch (r.opCode()) {
		case icpHit:
		case icpMiss:
		case icpMissNoFetch:
				finish(0);
				break;
		default:
				finish(errIcpRepCode);
	}
}

IcpClient *IcpCltXact::icpClient() {
	return theOwner->icpClient();
}

void IcpCltXact::finish(int errNo) {
	IcpXaction::finish(errNo);

	if (theReserv)
		icpClient()->cancel(theReserv);
	cancelAlarms();

	theOwner->notePeerAsked(this);
}
