
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_SERVER_REPRESENTATIVE_H
#define PG_SERVER_REPRESENTATIVE_H

// client side "representative" of an origin server 
// holds client side knowledge about a server

#include "xstd/Ring.h"
#include "xstd/NetAddr.h"
#include "base/UniqId.h"
#include "client/NidsStore.h"

class OLog;
class RepHdr;

class ServerRep {
	public:
		static int TheSharedWss;  // sum of all server wss's
		static void ConfigureNids(int prefetchLmt, int perReqMax);

	public:
		ServerRep(const NetAddr &anAddr, int hostIdx);

		const NetAddr &addr() const { return theAddr; }
		const UniqId &id() const { return theId; }
		int hostIdx() const { return theHostIdx; }

		bool haveNewOids() const { return !theNids.empty(); }
		bool haveFailedOids() const { return theFailedOids.count(); }
		int wss() const { return theWss; }
		int accesses() const { return theAccCount; }
		int lastRepPos() const { return theLastRepPos; }
		bool wasFull() const;

		int pendNidPack() const { return thePendNidPack; }
		int nidShortage() const;

		int newOid();
		int failedOid();
		int secOid() const { return theSecOid; }
		int hotSetPos() const { return theHotSetPos; }

		void noteFailedOid(int oid);
		void syncSrvInfo(const RepHdr &rep);
		void noteNoNewOids() { theNids.noteNoNids(); }

		void freezeWss();      // based on local info only

		void store(OLog &log) const;
		void load(ILog &log);

	protected:
		void syncSrvId(const UniqId &aSrvId);
		void syncWss(int wss);
		void syncNids(int startOid, int askCnt, int gotCnt);

	protected:
		static int TheNidsPrefetchDepth; // control new oid prefetching
		static int TheNidsPerReqMax;     // algorithm

	protected:
		NetAddr theAddr;
		UniqId theId;
		NidsStore theNids;
		Ring<int> theFailedOids;

		int theHostIdx;    // index into the HostMap
		int theSecOid;     // our idea of server's "secure" oid
		int theAccCount;   // accesses to this server
		int theLastRepPos; // position of the last reply received
		int thePendNidPack;// new oid pack number
		int theWss;        // number of oids in server's working set
		int theHotSetPos;  // as set by the server
};

#endif
