
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include "runtime/LogComment.h"
#include "runtime/polyBcastChannels.h"
#include "runtime/StatPhase.h"
#include "runtime/StatPhaseMgr.h"
#include "client/wssFreezers.h"


/* TimeWssFreezer */

TimeWssFreezer::TimeWssFreezer(Time aDelay): theDelay(aDelay) {
}

void TimeWssFreezer::start() {
	WssFreezer::start();
	sleepFor(theDelay);
}

void TimeWssFreezer::wakeUp(const Alarm &alarm) {
	AlarmUser::wakeUp(alarm);
	startFreeze();
}

void TimeWssFreezer::printGoal(ostream &os) const {
	os << "first " << theDelay << " of the test";
}


/* FillWssFreezer */

FillWssFreezer::FillWssFreezer(int aFillCount): theFillCount(aFillCount) {
	theChannels.append(TheXactEndChannel);
}

void FillWssFreezer::start() {
	WssFreezer::start();
	startListen();
}

void FillWssFreezer::noteXactEvent(BcastChannel *ch, const Xaction *) {
	if (ch == TheXactEndChannel && TheStatPhaseMgr->fillCnt() >= theFillCount) {
		stopListen();
		startFreeze();
	}
}

void FillWssFreezer::printGoal(ostream &os) const {
	os << "first " << theFillCount << " fill objects of the test";
}
