
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl.h"

#include "String.h"
#include "PglRec.h"
#include "PglDistrSym.h"
#include "PglStringSym.h"
#include "SocketSym.h"
#include "UniqIdSym.h"
#include "AgentSym.h"


AgentSym::AgentSym(const String &aType): HostsBasedSym(aType) {
	theRec->bAdd("Socket", "socket", new SocketSym);

	theRec->bAdd("string", "kind", 0);
	theRec->bAdd("uniq_id", "world", 0);
	theRec->bAdd("time_distr", "xact_think", 0);
	theRec->bAdd("int_distr", "pconn_use_lmt", 0);
	theRec->bAdd("time", "idle_pconn_tout", 0);
}

AgentSym::AgentSym(const String &aType, PglRec *aRec): HostsBasedSym(aType, aRec) {
}

bool AgentSym::isA(const String &type) const {
	return HostsBasedSym::isA(type) || type == "Agent";
}

SynSym *AgentSym::clone(const String &type) const {
	if (isA(type))
		return new AgentSym(this->type(), theRec->clone());
	return HostsBasedSym::clone(type);
}

String AgentSym::kind() const {
	return getString("kind");
}

UniqId AgentSym::world() const {
	SynSymTblItem *wi = 0;
	Assert(theRec->find("world", wi));
	return wi->sym() ?
		((const UniqIdSym&)wi->sym()->cast("uniq_id")).val() : UniqId();
}

SocketSym *AgentSym::socket() const {
	SynSymTblItem *si = 0;
	Assert(theRec->find("socket", si));
	Assert(si->sym());
	return &(SocketSym&)si->sym()->cast("Socket");
}

RndDistr *AgentSym::xactThink() const {
	return getDistr("xact_think");
}

RndDistr *AgentSym::pconnUseLmt() const {
	return getDistr("pconn_use_lmt");
}

Time AgentSym::idlePconnTimeout() const {
	return getTime("idle_pconn_tout");
}
