
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "pgl.h"

#include "PglNumSym.h"
#include "PglTimeSym.h"
#include "PglSizeSym.h"
#include "PglBwidthSym.h"

#include "gadgets.h"

static BigSize bytes2size(double bytes) { return BigSize::MBd(bytes/1024/1024); }


BwidthSym::BwidthSym(BigSize aSize, Time anInterval): ExpressionSym("bwidth"),
	theVal(::Bandwidth(aSize, anInterval)) {
}

BwidthSym::BwidthSym(double aVal): ExpressionSym("bwidth") {
	val(aVal);
}

bool BwidthSym::isA(const String &type) const {
	return ExpressionSym::isA(type) ||
		type == "Bwidth" || type == "bwidth";
}

SynSym *BwidthSym::clone(const String &type) const {
	if (isA(type))
		return new BwidthSym(theVal);
	return ExpressionSym::clone(type);
}

Bwidth BwidthSym::val() const {
	return theVal;
}

void BwidthSym::val(Bwidth aVal) {
	theVal = aVal;
}

ExpressionSym *BwidthSym::unOper(const String &op) const {
	if (*op == '+')
		return new BwidthSym(+theVal);
	else
	if (*op == '-')
		return new BwidthSym(-theVal);
	else
		return ExpressionSym::unOper(op);
}

ExpressionSym *BwidthSym::bnOper(const String &op, const SynSym &s) const {
	if (op.len() != 1)
		return ExpressionSym::bnOper(op, s);

	if (BwidthSym *rs = (BwidthSym*)s.clone("bwidth"))
		return operBB(op, rs, s);

	if (TimeSym *ts = (TimeSym*)s.clone("time"))
		return operBT(op, ts, s);

	if (NumSym *ns = (NumSym*)s.clone("float"))
		return operBN(op, ns, s);

	return ExpressionSym::bnOper(op, s);
}

// bwidth op bwidth
ExpressionSym *BwidthSym::operBB(const String &op, BwidthSym *rs, const SynSym &s) const {
	double otherVal = rs->val();
	delete rs;

	if (*op == '/')
		checkDenom(otherVal);

	switch (*op) {
		case '+': return new BwidthSym(val() + otherVal);
		case '-': return new BwidthSym(val() - otherVal);
		case '/': return new NumSym(val() / otherVal);
		default:  return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *BwidthSym::operBT(const String &op, TimeSym *ts, const SynSym &s) const {
	Time otherInterval = ts->val();
	delete ts;

	switch (*op) {
		case '*': return new SizeSym(bytes2size(BytesPerSec(theVal)*otherInterval.secd()));
		default: return ExpressionSym::bnOper(op, s);
	}
}

ExpressionSym *BwidthSym::operBN(const String &op, NumSym *ns, const SynSym &s) const {
	double otherFactor = ns->val();
	delete ns;
	
	if (*op == '/')
		checkDenom(otherFactor);

	switch (*op) {
		case '*': return new BwidthSym(theVal*otherFactor);
		case '/': return new BwidthSym(theVal/otherFactor);
		default: return ExpressionSym::bnOper(op, s);
	}
}

ostream &BwidthSym::print(ostream &os, const String &) const {
	return os << bytes2size(theVal*1) << "/sec";
}


Bwidth Bandwidth(BigSize sz, Time tm) {
	return sz.byted()/tm.secd();
}
