
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_HOST_MAP_CLASS_H
#define PG_HOST_MAP_CLASS_H

#include "Array.h"
#include "NetAddr.h"

class ContentSel;

// Host configuration record
class HostCfg {
	public:
		HostCfg(const NetAddr &anAddr);

	public:
		NetAddr theAddr;
		ContentSel *theContent; // can be shared among HostCfgs!
};

// a NetAddr <-> Host configuration map
// add-only operation
class HostMap {
	public:
		HostMap(int aCapacity); // cap may be adjusted a bit
		~HostMap();

		int capacity() const { return theIndex.capacity(); }
		int count() const { return theCount; }

		HostCfg *at(int idx); // idx may be out of bounds
		HostCfg *at(const NetAddr &addr);

		bool find(const NetAddr &addr, int &idx) const;
		HostCfg *addAt(int idx, const NetAddr &addr);

	protected:
		bool endSearch(const NetAddr &addr, int idx, bool &res) const;
		int hash0(const NetAddr &addr) const;
		int hash1(const NetAddr &addr) const;

	protected:
		Array<HostCfg*> theIndex;
		int theCount;  // entries in the hash
};

extern HostMap *TheHostMap; // XXX: should be in globals, but ObjId needs it

#endif
