
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#include "base/polygraph.h"

#include <math.h>

#include "runtime/StatPhase.h"
#include "runtime/TransFactor.h"

TransFactor::TransFactor(): thePhase(0), theBeg(1), theEnd(1), theK(0) {
}

// note: simply left some values unchanged on failure
// the caller will check and complain
void TransFactor::configure(const StatPhase *aPhase, double beg, double end, const TransFactor *prev) {
	Assert(!thePhase && aPhase);
	thePhase = aPhase;

	if (beg >= 0)
		theBeg = beg;
	else
	if (prev) // use the previous end factor if needed
		theBeg = prev->theEnd;

	theEnd = end >= 0 ? end : theBeg;	

	const double fdiff = theEnd - theBeg;
	
	if (fdiff && !goal())
		return;

	// cache often needed value
	// note: duration takes priority over xact count if both are given
	theK = 
		goal().duration() > 0 ?
			fdiff/goal().duration().secd() :
		(goal().xactCnt() > 0 ?
			fdiff/goal().xactCnt() : 0);
}

const GoalRec &TransFactor::goal() const {
	return thePhase->goal();
}

double TransFactor::current(Time lastTm, int lastCnt) const {
	const double t = goal().duration() > 0 ?
		Max((lastTm - thePhase->startTime()).secd(), 0.0):
		(double)lastCnt;

	const double lastF = theK*t + theBeg;
	const double d = lastF*lastF + 4*theK;
	const double newF = d >= 0 ? (sqrt(d) + lastF)/2 : (lastF/2);

	// make sure load factor stays between beg and end
	return theBeg <= theEnd ?
		MiniMax(theBeg, newF, theEnd):
		MiniMax(theEnd, newF, theBeg);
}


