
/*
 * Web Polygraph                        http://polygraph.ircache.net/
 * ------------------------------------------------------------------
 *
 *  Web Polygraph is copyrighted (C) 1998-2000 by The Regents of the
 *  University of California. All rights reserved. See the COPYRIGHT
 *  file for details.
 *
 */

#ifndef PG_SERVER_HISTORY_H
#define PG_SERVER_HISTORY_H

#include "xstd/BitMask.h"
#include "runtime/HotSet.h"

class OLog;
class CltGroupRep;
class CltGroupMap;
class ReqHdr;

// holds information about previously requested objects
// only objects that belong to "public" server area are traced

class SrvHistory {
	public:
		SrvHistory();
		~SrvHistory();

		void window(int capacity);

		int newOids(int needCnt, int &resCnt);
		int secOid() const { return theSecOid; }
		int wss() const { return theWss; }

		void noteOid(int oid); // note requested oid
		void noteWss(int wss); // note clients's wss
		CltGroupRep *syncCltGroupInfo(const ReqHdr &req);

		int hotSetPos() { return theHotSet.position(theSecOid, theWss); }

		void store(OLog &log) const;

	protected:
		int newOid();

		inline void requested(int oid, bool be);
		inline bool requested(int oid) const;

	protected:
		CltGroupMap *theCltGroupMap; // per-client-group info index

		BitMask theAdvWin; // requested advertised oids, sliding window
		int theSecOid;     // last secured oid; all (<= oids) were requested
		int theAdvOid;     // last advertized oid (to some robot)
		int thePendCnt;    // number of not requested advertised oids

		int theWss;	   // public working set size (collected from clients)
		HotSet theHotSet;  // hot set model

		Time theSecTime;   // time when sec oid was updated
};

inline
OLog &operator <<(OLog &log, const SrvHistory &sh) {
	sh.store(log);
	return log;
}

/* inlined methods */

void SrvHistory::requested(int oid, bool be) {
	if (be) {
		theAdvWin.beSet(oid % theAdvWin.capacity());
		--thePendCnt;
	} else {
		theAdvWin.beClear(oid % theAdvWin.capacity());
		++thePendCnt;
	}
}

bool SrvHistory::requested(int oid) const {
	return theAdvWin.isSet(oid % theAdvWin.capacity());
}


#endif
